package net.knarcraft.bookswithoutborders.command;

import net.knarcraft.bookswithoutborders.BooksWithoutBorders;
import net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig;
import net.knarcraft.bookswithoutborders.utility.EconomyHelper;
import org.bukkit.Material;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.command.PluginCommand;
import org.bukkit.command.TabExecutor;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;

import static net.knarcraft.bookswithoutborders.BooksWithoutBorders.sendErrorMessage;
import static net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig.getCommandColor;
import static net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig.getSuccessColor;

/**
 * Command executor for the books without borders (bwb) command
 */
public class CommandBooksWithoutBorders implements TabExecutor {

    @Override
    public boolean onCommand(CommandSender sender, @NotNull Command command, @NotNull String label, @NotNull String[] args) {
        sender.sendMessage(getCommandColor() + "[] denote optional parameters");
        sender.sendMessage(getCommandColor() + "<> denote required parameters");
        sender.sendMessage(getCommandColor() + "{} denote required permission");
        sender.sendMessage(getCommandColor() + "In some cases, commands with required parameters can be called with no parameters");
        if (sender instanceof Player) {
            showPlayerCommands(sender);
        } else {
            showConsoleCommands(sender);
        }
        return true;
    }

    /**
     * Shows all commands available to the console
     *
     * @param sender <p>The console which sent the command</p>
     */
    private void showConsoleCommands(CommandSender sender) {
        sender.sendMessage(getCommandColor() + "Commands:");
        showCommandInfo("deletePublicBook", sender);
        showCommandInfo("givePublicBook", sender);
        showCommandInfo("reload", sender);
    }

    /**
     * Shows all commands available to the sending player
     *
     * @param sender <p>The player which sent the command</p>
     */
    private void showPlayerCommands(CommandSender sender) {
        //Lists all commands
        Material bookPriceType = BooksWithoutBordersConfig.getBookPriceType();
        double bookPriceQuantity = BooksWithoutBordersConfig.getBookPriceQuantity();
        if (BooksWithoutBordersConfig.booksHavePrice()) {
            if (bookPriceType != Material.AIR) {
                sendErrorMessage(sender, "[" + (int) bookPriceQuantity + " " + bookPriceType.toString() +
                        "(s) are required to create a book]");
            } else {
                sendErrorMessage(sender, "[" + EconomyHelper.getEconomy().format(bookPriceQuantity) +
                        " is required to create a book]");
            }
        }
        sender.sendMessage(getCommandColor() + "Commands:");

        showCommandInfo("copyBook", sender);
        showCommandInfo("decryptBook", sender);
        showCommandInfo("deleteBook", sender);
        showCommandInfo("deletePublicBook", sender);
        showCommandInfo("encryptBook", sender);
        showCommandInfo("formatBook", sender);
        showCommandInfo("giveBook", sender);
        showCommandInfo("givePublicBook", sender);
        showCommandInfo("groupEncryptBook", sender);
        showCommandInfo("loadBook", sender);
        showCommandInfo("loadPublicBook", sender);
        showCommandInfo("reload", sender);
        showCommandInfo("saveBook", sender);
        showCommandInfo("savePublicBook", sender);
        showCommandInfo("setAuthor", sender);
        showCommandInfo("setBookPrice", sender);
        showCommandInfo("setLore", sender);
        showCommandInfo("setTitle", sender);
        showCommandInfo("unsignBook", sender);
    }

    /**
     * Shows information about the given command
     *
     * @param command <p>The command to get information about</p>
     * @param sender  <p>The sender asking to see command info</p>
     */
    private void showCommandInfo(String command, CommandSender sender) {
        PluginCommand pluginCommand = BooksWithoutBorders.getInstance().getCommand(command);
        if (pluginCommand != null) {
            String permission = pluginCommand.getPermission();
            if (permission == null || sender.hasPermission(permission)) {
                String commandInfo = "\n" + getCommandColor() + pluginCommand.getUsage().replace("<command>",
                        pluginCommand.getName()) + ": " + getSuccessColor() + pluginCommand.getDescription();
                if (sender.hasPermission("bookswithoutborders.admin")) {
                    if (permission == null) {
                        permission = "None";
                    }
                    commandInfo += getCommandColor() + " {" + permission + "}";
                }
                sender.sendMessage(commandInfo);
            }
        }
    }

    @Override
    public List<String> onTabComplete(@NotNull CommandSender sender, @NotNull Command command, @NotNull String alias, @NotNull String[] args) {
        return new ArrayList<>();
    }

}
